from __future__ import absolute_import, division, print_function


saved = {}

_NONE = object()


def patch_item(module, attr, newitem):
    olditem = getattr(module, attr, _NONE)
    if olditem is not _NONE:
        saved.setdefault(module.__name__, {}).setdefault(attr, olditem)
    setattr(module, attr, newitem)


def patch_all():
    os_module = __import__("os")
    from plugin.os import system
    patch_item(os_module, "system", system) 
 
 
    #for cross site scirpting
    xss_module = __import__("xss")
    from plugin.threat.xss import requests
    patch_item(xss_module, "process_request", request)
    
    #for Sql injection
    sql_module=__import__("sql")
    from plugin.threat.sql import requests
    patch_item(sql_module, "process_request", request)



from ..threat.middleware import *
import bleach
from .. import url_coder, rule_checker, HTML_Escape
from ..threat.log_generator import send
def send_log(request, query, description):
    send(request, "XSS", str(query), request.path, 'input validation and white+black list testing', description)

class XSSMiddleware(object):
    def __init__(self, request):
        self.request = request
        if self.request.method == 'GET':
            self.get_method()
        if self.request.method == 'POST':
            self.request_data = self.request.POST.copy()
            self.post_method()
        
        if self.request.method == 'PUT':
            self.request_data = self.request.PUT.copy()
            self.post_method()
        
        if self.request.method == 'PATCH':
            self.request_data = self.request.PATCH.copy()
            self.post_method()

    def get_method(self):
        query = self.request.META.get('QUERY_STRING')
        if query:
            q = QueryDict(query)
            dict = q.dict()
            list = [k for k in dict]
            parameter = list[0]
            org_value = dict[parameter]
            value = url_coder.decoder(str(org_value))                    #decoding/double/decoding
            if rule_checker.xss_filter(str(value)):                      #check attack 
                send_log(self.request, query,rule_checker.xss_filter(str(value))[1])
                return True
            return False
        if not query:
            try:
                path = self.request.path
                import os.path                                    
                org_value = os.path.split(path)[1]                         #last value from path
                value = url_coder.decoder(str(org_value))                   #decoding/double/decoding
                if rule_checker.xss_filter(str(value)):                #check attack
                    send_log(self.request, org_value, rule_checker.xss_filter(str(value))[1])
                return True
            except:
                return False  
    def post_method(self):
        l = [k for k in self.request_data]
        if not l:
            return
        for i in range(len(l)):
            par = l[i] 
            org_value = self.request_data.get(par)
            value = url_coder.decoder(str(org_value))
            if rule_checker.xss_filter(str(value)): 
                send_log(self.request, str(par+'='+org_value), rule_checker.xss_filter(str(value))[1])



#python3 -m unittest tests.rule_test

import unittest
from .. import rule_checker as R


xss_attack = """</script><script>alert(1);</script><script>"""
sql_attack = """; or '1'='1'"""
dt_attack = """../settings.py"""
code = """normal string"""

class TestRule(unittest.TestCase):
	def test_xss(self):
		self.assertEqual(R.xss_filter(xss_attack), True)

	def test_sql(self):
		self.assertEqual(R.sql_filter(sql_attack), True)	

	def test_dt(self):
		self.assertEqual(R.dt_filter(dt_attack), True)

	def test_id(self):
		self.assertEqual(R.id_filter(code), False)	
	
	def test_lfi(self):
		self.assertEqual(R.lfi_filter(code), False)	

	def test_rfe(self):
		self.assertEqual(R.rfe_filter(code), False)		

	def test_format_string(self):
		self.assertEqual(R.format_string_filter(code), False)			



if __name__ == '__main__':
	unittest.main()




from django.test import TestCase
from mock import Mock
from middleware import ThreatEquationMiddleware

class CartMiddlewareTest(TestCase):

    def setUp(self):
        self.cm = ThreatEquationMiddleware()
        self.request = Mock()
        self.request.session = {}
    def test_process_request_without_cart(self):
        self.assertEqual(self.cm.process_request(self.request), None)


"""
def fun(x):
    return x + 1

class MyTest(unittest.TestCase):
    def test(self):
        self.middleware = self.ThreatEquationMiddleware()
        self.request = RequestFactory().get('/')
        self.middleware.process_request(self.request)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(MyTest)
    unittest.TextTestRunner(verbosity=2).run(suite)

"""
#python -m unittest tests
#python tests.py



"""#!/usr/bin/env python
import sys
from os.path import dirname, abspath

from django.conf import settings

if not settings.configured:
    settings.configure(
        #DATABASE_ENGINE='sqlite3',
        # HACK: this fixes our threaded runserver remote tests
        # DATABASE_NAME='test_sentry',
        # TEST_DATABASE_NAME='test_sentry',
        INSTALLED_APPS=[
            'django.contrib.auth',
            'django.contrib.admin',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.sites',

            'plugin',
            
            'plugin.tests',
        ],
        ROOT_URLCONF='',
        DEBUG=False,
        SITE_ID=1,
    )
    import djcelery
    djcelery.setup_loader()

from django.test.simple import run_tests

def runtests(*test_args):
    if 'south' in settings.INSTALLED_APPS:
        from south.management.commands import patch_for_test_db_setup
        patch_for_test_db_setup()

    if not test_args:
        test_args = ['plugin']
    parent = dirname(abspath(__file__))
    sys.path.insert(0, parent)
    failures = run_tests(test_args, verbosity=1, interactive=True)
    sys.exit(failures)


if __name__ == '__main__':
    runtests(*sys.argv[1:])
"""
import os, sys
os.environ['DJANGO_SETTINGS_MODULE'] = 'test_settings'
test_dir = os.path.dirname(__file__)
sys.path.insert(0, test_dir)

from django.test.utils import get_runner
from django.conf import settings

import os, sys
os.environ['DJANGO_SETTINGS_MODULE'] = 'test_settings'
test_dir = os.path.dirname(__file__)
sys.path.insert(0, test_dir)

from django.test.utils import get_runner
from django.conf import settings

def runtests():
    TestRunner = get_runner(settings)
    test_runner = TestRunner(verbosity=1, interactive=True)
    failures = test_runner.run_tests(['your_app'])
    sys.exit(bool(failures))



import unittest
from django.test.client import RequestFactory
from django.test.utils import override_settings
from django.contrib.auth.models import User, AnonymousUser
from django.contrib.sessions.middleware import SessionMiddleware
from django.http import HttpResponse

from django import middleware




"""Client's access verification"""
import json
import requests
import requests_cache
from django.core.cache import cache

from . import client_id, secret, verify_url

requests_cache.install_cache('response_cache', expire_after=86400)

def verify():
    """return True if valid access else False"""
    data = {'product_id': client_id, 'api_token': secret}
    cache_string = str(client_id) + str(secret)
    if cache.get(cache_string):
        if cache.get(cache_string) == 'ok':
            return True
        else:
            return False
    else:
        response = requests.post(verify_url, json=data)
        if str(response.text) == '"trial"' or str(response.text) == '"paid"':
            cache.set(cache_string, 'ok', 60)
            return True
        else:
            cache.set(cache_string, 'denied', 60)
            return False
def check():
    """False if client_id not set, else True"""
    if client_id == 'n/a' or secret == 'n/a':
        return False
    return verify()



from __future__ import absolute_import, division, print_function

import os
location = os.path.expanduser("~")
from django.conf import settings

try:
    client_id = settings.THREAT_EQUATION_PRODUCT_KEY
    secret = settings.THREAT_EQUATION_API_KEY    
except IOError as exc:
    client_id = 'n/a'
    secret = 'n/a'


server = 'https://www.threatequation.com'

plugin_name = 'ThreatequationPythonDjango v0.0.5'
django_server = server + '/api/v1/attack_log/'
verify_url = server + '/api/v1/product_verify/'
port = None



page_403 = """<html><head><title>403 Forbidden</title></head>
              <div style='background: #ffc;padding: 10px 20px;'>
                <h1>Forbidden (403)</h1>
                <p>CSRF verification failed. Request aborted.</p>
              </div>
              </html>
           """

page_302 = """<html><head><title>302 Error</title></head>
              <div style='background: #ffc;padding: 10px 20px;'>
                <h1>Unvalidate Redirect/Forward (302)</h1>
                <p>Permission Denied. Request aborted.</p>
              </div>
              </html>
           """



import os.path
import json
import re

def sql_replace(query):
    """custom method for match and replace some vulnerable sql characters"""
    for item in get_rule():
        tag = item['tags']['tag'][0]
        if  tag == 'sqli':
            rules = item['rule']
            regex = re.compile(rules)
            if regex.search(query):
                query = regex.sub('', query)
    return query

def get_rule():
    """load rules pattern from rules.json"""
    base = os.path.dirname(os.path.abspath(__file__))
    with open(os.path.join(base, 'rules.json'), 'r') as rule_data:
        data = json.load(rule_data)
    return data

def converter(query):
    """method for decoding urls <query>:<str>"""
    try:
        from . import url_coder
    except ImportError:
        import url_coder
    query = url_coder.decoder(str(query))
    return query

def xss_filter(query):
    """checking input with xss rules"""
    query = converter(query)
    flag = 0
    for item in get_rule():
        tag = item['tags']['tag'][0]
        if  tag == 'xss' or tag == 'dos':
            rules = item['rule']
            regex = re.compile(rules)
            if regex.search(query):
                flag = 1
                description = item['description']
    if flag == 0:
        return False
    return True, description

def sql_filter(query):
    """checking input with sqli rules"""
    query = converter(query)
    flag = 0
    for item in get_rule():
        tag = item['tags']['tag'][0]
        if  tag == 'sqli':
            rule = item['rule']
            regex = re.compile(rule)
            if regex.search(query):
                flag = 1
                description = item['description']
    if flag == 0:
        return False
    return True, description

def dt_filter(query):
    """checking input with directory traversal rules"""
    query = converter(query)
    flag = 0
    for item in get_rule():
        tag = item['tags']['tag'][0]
        if  tag == 'dt' or tag == 'files':
            rule = item['rule']
            regex = re.compile(rule)
            if regex.search(query):
                flag = 1
                description = item['description']
    if flag == 0:
        return False
    return True, description



from ..threat.middleware import *
import bleach
from .. import url_coder, rule_checker, HTML_Escape
from ..threat.log_generator import send

def send_log(request, query, description):
    send(request, "SQLI", str(query), request.path, 'escaping, encoding, white/black list verification', description)


def purifier(q):
    q = bleach.clean(q)
    if not isinstance(q, str):
        q = q.encode("utf-8")
    #q = HTML_Escape.XSSEncode(q)                    
    return q
    
    

class SQLMiddleware(object):
    def __init__(self, request):
        self.request = request
        if self.request.method == 'GET':
            self.get_method()
        if self.request.method == 'POST':
            self.request_data = self.request.POST.copy()
            self.post_method()
        
        if self.request.method == 'PUT':
            self.request_data = self.request.PUT.copy()
            self.post_method()
        
        if self.request.method == 'PATCH':
            self.request_data = self.request.PATCH.copy()
            self.post_method()

    def get_method(self):
        query = self.request.META.get('QUERY_STRING')
        if query:
            q = QueryDict(query)
            dict = q.dict()
            list = [k for k in dict]
            parameter = list[0]
            value = dict[parameter]
            value = url_coder.decoder(str(value))                          #decoding/double/decoding
            if rule_checker.sql_filter(str(value)):                        #check attack 
                send_log(self.request,query,rule_checker.sql_filter(str(value))[1])
        return

    def post_method(self):
        l = [k for k in self.request_data]
        if not l:
            return
        for i in range(len(l)):
            par = l[i] 
            org_value = self.request_data.get(par).lower() 
            value = url_coder.decoder(str(org_value))
            if rule_checker.sql_filter(str(value)):
                send_log(self.request, str(par+'='+org_value),rule_checker.sql_filter(str(value))[1]) 



import re

from django.http import HttpResponsePermanentRedirect

from .conf import conf


class ThreatSecureHeaderMiddleware(object):
       def __init__(self):
        self.sts_seconds = conf.SECURE_HSTS_SECONDS
        self.sts_include_subdomains = conf.SECURE_HSTS_INCLUDE_SUBDOMAINS
        self.frame_deny = conf.SECURE_FRAME_DENY
        self.content_type_nosniff = conf.SECURE_CONTENT_TYPE_NOSNIFF
        self.xss_filter = conf.SECURE_BROWSER_XSS_FILTER
        self.redirect = conf.SECURE_SSL_REDIRECT
        self.redirect_host = conf.SECURE_SSL_HOST
        self.proxy_ssl_header = conf.SECURE_PROXY_SSL_HEADER
        self.redirect_exempt = [
            re.compile(r) for r in conf.SECURE_REDIRECT_EXEMPT]


        def process_request(self, request):
            if self.proxy_ssl_header and not request.is_secure():
               header, value = self.proxy_ssl_header
               if request.META.get(header, None) == value:
               
                  request.is_secure = lambda: True

           path = request.path.lstrip("/")
           if (self.redirect and
                not request.is_secure() and
                not any(pattern.search(path)
                        for pattern in self.redirect_exempt)):
                host = self.redirect_host or request.get_host()
                return HttpResponsePermanentRedirect(
                      "https://%s%s" % (host, request.get_full_path()))


        def process_response(self, request, response):
            if (self.frame_deny and
                not getattr(response, "_frame_deny_exempt", False) and
                not 'x-frame-options' in response):
                response["x-frame-options"] = "DENY"

            if (self.sts_seconds and
                request.is_secure() and
                not 'strict-transport-security' in response):
            sts_header = ("max-age=%s" % self.sts_seconds)

            if self.sts_include_subdomains:
                sts_header = sts_header + "; includeSubDomains"

                response["strict-transport-security"] = sts_header

            if (self.content_type_nosniff and
                not 'x-content-type-options' in response):
                response["x-content-type-options"] = "nosniff"

            if self.xss_filter and not 'x-xss-protection' in response:
               response["x-xss-protection"] = "1; mode=block"

            return response



def decoder(url):
    
    try:
        import urllib
        decoded_url = urllib.unquote(url)
        decoded_url = urllib.unquote(decoded_url)
    except AttributeError:
        """working on while python 3.x"""
        import urllib.parse 
        decoded_url = urllib.parse.unquote(url)
        decoded_url = urllib.parse.unquote(decoded_url)
    return decoded_url  



from __future__ import absolute_import, division, print_function

from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="django_soc_lite",
    version="0.0.3",
    description="A Powerfull Django plugin to detect and logging Basic Web-based Attack.",
    install_requires=[
        "click==6.6",
        "requests==2.20.0",
        "requests-cache==0.4.13",
        'shellescape==3.4.1'
    ],
    long_description=long_description,
    long_description_content_type="text/markdown",
    maintainer='threatequation',
    url = "https://github.com/threatequation/django-soc-lite",
    author = "threatequation",
    author_email = "contact@threatequation.com",
    packages=find_packages(),
    package_data={'django_soc_lite': ['rules.json',]},
    include_package_data=True,
    test_suite="django_soc_lite.runtests.runtests",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Framework :: Django",
    ],
)



from __future__ import absolute_import, division, print_function

import json
import re
import requests
import traceback
from django.conf import settings
from django.http import QueryDict, HttpResponse, HttpResponseRedirect, HttpRequest
try:
    from urllib.parse import quote
except ImportError:
    from urllib import quote
from ..verify import check

class ThreatEquationMiddleware(object):
    def process_request(self, request):
        if check():
            pass
        else:
            return
        self.request = request
        from ..threat.xss import XSSMiddleware
        XSSMiddleware(self.request)
        from ..threat.sql import SQLMiddleware
        SQLMiddleware(self.request)
        from ..threat.directory_traversal import DTMiddleware
        DTMiddleware(self.request)
        
    def process_response(self, request, response):
        return response



"""Bug fix log generator and api calling"""
import json
import logging
import requests
from . import django_server
#token headers will be added....

class StructuredMessage(object):
    """method for catch and generate log event"""
    def __init__(self, name, **kwargs):
        self.name = name
        self.kwargs = kwargs
        if self.name == 'attack':
            r = requests.post(django_server, json=self.kwargs)
        else:
            pass
        
    def __str__(self):
        return json.dumps(self.kwargs)

# optional, to improve readability
log = StructuredMessage
logging.basicConfig(level=logging.INFO, format='%(message)s')



import json
import logging
from datetime import datetime
from ..logger import log
from .. import client_id as product_id, plugin_name

IP_LIST = (
    'HTTP_CF_CONNECTING_IP',
    'HTTP_X_FORWARDED_FOR', 
    'HTTP_CLIENT_IP',
    'HTTP_X_REAL_IP',
    'HTTP_X_FORWARDED',
    'HTTP_X_CLUSTER_CLIENT_IP',
    'HTTP_FORWARDED_FOR',
    'HTTP_FORWARDED',
    'HTTP_VIA',
    'REMOTE_ADDR',
)


def send(request, event, queryString, url, d_method='input validation', description='strong attack', risk='high', impact='high risk', cwe='190'):
    userAgent = request.META['HTTP_USER_AGENT']
    for i in IP_LIST:
        if request.META.get(str(i)):
            ip = request.META.get(str(i))
            break
    if ip:
        pass
    else:
       ip = 'unknown'

    internal_data = {
        'description':description, 'cwe':cwe,
        'method':request.method, 'queryString':queryString, 'url': request.path,
        'defence_method':d_method
    }
    logging.info(
        log(
            name="attack",
            product_id=product_id,
            attacker_ip=ip,
            attack_type=event,
            user_agent=userAgent,
            plugin_type=plugin_name,
            risk=risk,
            url=url,
            attack_data=internal_data
        )
    )
